import unittest

from eosiopy import NodeNetwork, RawinputParams, EosioParams


class EosioPyTestCase(unittest.TestCase):
    def test_get_info(self):
        info = NodeNetwork.get_info()
        self.assertTrue(isinstance(info, dict))
        block = NodeNetwork.get_block(1)
        self.assertTrue(isinstance(block, dict))

    def test_newaccount(self):
        raw = RawinputParams("newaccount", {"creator": "eosio", "name": "aeztcnjmhege", "owner": {"threshold": 1,
                                                                                                  "keys": [{
                                                                                                      "key": "EOS5XhYEWPJH2P77rh6dwMURsgjFodjXqTadSf1tNZAHVhnBpWyNu",
                                                                                                      "weight": 1}],
                                                                                                  "accounts": [],
                                                                                                  "waits": []},
                                            "active": {"threshold": 1, "keys": [
                                                {"key": "EOS7EZgfh13yVxaXuzH12cC2Yru7Wv1JNNNbxnSdZQXNX2hWAyBTm",
                                                 "weight": 1}], "accounts": [], "waits": []}}, "eosio", "eosio@active")
        eosiop_arams = EosioParams(raw.params_actions_list, "5KQwrPbwdL6PhXujxW37FSSQZ1JiwsST4cqQzDeyXtP79zkvFD3")
        net = NodeNetwork.push_transaction(eosiop_arams.trx_json)
        self.assertTrue(isinstance(net, dict))
        self.assertNotEqual(net.get("transaction_id", None), None)


if __name__ == '__main__':
    unittest.main()



import json
import time

from eosiopy import eosio_config
from eosiopy.nodenetwork import NodeNetwork
from eosiopy.packedtransaction import PackedTransaction
from eosiopy.sign import sign


class EosioParams(object):
    def __init__(self, params_actions_list, wif=None):
        self.get_info_block()
        self.params = dict()

        # actions = list()
        # actions.append(params_dict)
        tmp_dict = dict()
        tmp_dict["actions"] = params_actions_list

        self.params.setdefault("transaction", tmp_dict)
        self.params.setdefault("ref_block_num", self.info_block["last_irreversible_block_num"])
        self.params.setdefault("ref_block_prefix", self.info_block["ref_block_prefix"])
        self.params.setdefault("delay_sec", 0)
        self.params.setdefault("max_kcpu_usage", 0)
        self.params.setdefault("max_net_usage_words", 0)
        self.params.setdefault("expiration", self.get_expiration())
        self.params.setdefault("signatures", list())
        p = json.dumps(self.params)
        self.packed()
        if wif:
            self.sign(wif)

    def get_info_block(self):
        self.info_block = NodeNetwork.get_info_block()

    def get_expiration(self):
        return int(time.time() + eosio_config.expiration)

    def packed(self):
        self.params["packed_trx"] = PackedTransaction(self.params, self.info_block["chain_id"])

    def sign(self, wif):
        signatures = list()
        signatures.append(sign(wif, self.params["packed_trx"].get_digest_signature()))
        self.params["signatures"] = signatures

    @property
    def trx_json(self):
        data = dict()
        data["compression"] = "none"
        data["packed_context_free_data"] = "00"
        data["packed_trx"] = self.params["packed_trx"].bytes_list.hex()
        data["signatures"] = self.params["signatures"]
        return data



import logging

import sys


def getLogger(name=None, filename=None):
    formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s')
    logger = logging.getLogger(name)
    sh = logging.StreamHandler(sys.stdout)
    if filename:
        fh = logging.FileHandler(filename)
    else:
        fh = logging.FileHandler('bitty.log')
    fh.setFormatter(formatter)
    sh.setFormatter(formatter)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(fh)
    logger.addHandler(sh)
    return logger



from .config import eosio_config
from .eosioparams import EosioParams
from .nodenetwork import NodeNetwork
from .rawinputparams import RawinputParams
from .sign import sign

VERSION = (0, 1, 3)
__version__ = VERSION
__versionstr__ = '.'.join(map(str, VERSION))

__all__ = ['VERSION', 'config', 'eosioparams', 'nodenetwork', 'packedtransaction',
           'rawinputparams', 'sign', 'utils']

# from eosiopy import eosio_config, RawinputParams, EosioParams, NodeNetwork
#
# eosio_config.url="http://127.0.0.1"
# eosio_config.port=8888
# raw = RawinputParams("newaccount", {"creator":"eosio","name":"aeztcnjmhege","owner":{"threshold":1,"keys":[{"key":"EOS5XhYEWPJH2P77rh6dwMURsgjFodjXqTadSf1tNZAHVhnBpWyNu","weight":1}],"accounts":[],"waits":[]},"active":{"threshold":1,"keys":[{"key":"EOS7EZgfh13yVxaXuzH12cC2Yru7Wv1JNNNbxnSdZQXNX2hWAyBTm","weight":1}],"accounts":[],"waits":[]}}, "eosio", "eosio@active")
# eosiop_arams=EosioParams(raw.params_actions_list,"5KQwrPbwdL6PhXujxW37FSSQZ1JiwsST4cqQzDeyXtP79zkvFD3")
# net=NodeNetwork.push_transaction(eosiop_arams.trx_json)
# print(net)



from eosiopy.utils import type_name_to_long


class PackedTransaction(object):
    bytes_list = bytearray()

    def __init__(self, eosio_params, chain_id):
        self.chain_id = chain_id
        params = eosio_params
        self.push_int(params["expiration"] & 0xFFFFFFFF)
        self.push_short(params["ref_block_num"] & 0xFFFF)
        self.push_int(params["ref_block_prefix"] & 0xFFFFFFFF)
        self.push_variableUInt(params["max_net_usage_words"])
        self.push_variableUInt(params["max_kcpu_usage"])
        self.push_variableUInt(params["delay_sec"])
        self.push_actiones(list())  # TODO packfreedata
        self.push_actiones(params["transaction"]["actions"])
        self.push_variableUInt(0)  # TODO packexdata

    def push_base(self, val, iteration):
        for i in iteration:
            self.bytes_list.append(0xFF & val >> i)

    def push_short(self, val):
        self.push_base(val, range(0, 9, 8))

    def push_int(self, val):
        self.push_base(val, range(0, 25, 8))

    def push_long(self, val):
        self.push_base(val, range(0, 57, 8))

    def push_char(self, val):
        self.bytes_list.append(int(val))

    def push_variableUInt(self, val):
        b = int((val) & 0x7f)
        val = val >> 7
        b = b | (((val > 0) if 1 else 0) << 7)
        self.push_char(b)
        while val:
            b = int((val) & 0x7f)
            val = val >> 7
            b = b | (((val > 0) if 1 else 0) << 7)
            self.push_char(b)

    def push_actiones(self, val_list):
        self.push_variableUInt(len(val_list))
        for i in val_list:
            self.push_long(type_name_to_long(i["account"]))
            self.push_long(type_name_to_long(i["name"]))
            self.push_permission(i["authorization"])
            if i["data"]:
                self.push_data(i["data"])
            else:
                self.push_variableUInt(0)

    def push_permission(self, val_list):
        self.push_variableUInt(len(val_list))
        for i in val_list:
            self.push_long(type_name_to_long(i["actor"]))
            self.push_long(type_name_to_long(i["permission"]))

    def push_data(self, val):
        bytes = bytearray.fromhex(val)
        self.push_variableUInt(len(bytes))
        for i in bytes:
            self.bytes_list.append(i)

    def push_transaction_extensions(self, val_list):
        self.push_variableUInt(len(val_list))

    def get_digest_signature(self):
        byte_array = bytearray.fromhex(self.chain_id)
        byte_array.extend(self.bytes_list)
        free_array = bytearray([0 for n in range(32)])
        byte_array.extend(free_array)
        return byte_array



import json
from collections import namedtuple

from eosiopy.config import eosio_config


def _json_object_hook(d):
    return namedtuple('X', d.keys())(*d.values())


def json2obj(data):
    return json.loads(data, object_hook=_json_object_hook)


def char_to_symbol(c):
    if (ord(c) >= ord('a') and ord(c) <= ord('z')):
        return chr(((ord(c) - ord('a')) + 6))

    if (ord(c) >= ord('1') and ord(c) <= ord('5')):
        return chr(((ord(c) - ord('1')) + 1))
    return chr(0)


def type_name_to_long(type_name):
    if type_name == None or type_name == "":
        return 0
    c = 0
    value = 0
    type_name_len = len(type_name)
    for i in range(eosio_config.MAX_NAME_IDX + 1):
        if (i < type_name_len and i <= eosio_config.MAX_NAME_IDX):
            c = ord(char_to_symbol(type_name[i]))
        if (i < eosio_config.MAX_NAME_IDX):
            c &= 0x1f
            c <<= 64 - 5 * (i + 1)
        else:
            c &= 0x0f
        value |= c
    return value



import ctypes
import hashlib
from ctypes import *

import base58
import pkg_resources

from eosiopy.exception import CantFindRecId, CantFindResInPack
from eosiopy.exception import IllegalKey


def get_res_path(name):
    for i in pkg_resources.resource_listdir(__name__, "../"):
        if i.find(name) >= 0:
            return i


def get_private_ket_by_wif(wif):
    eos_byte_array = bytearray(base58.b58decode(wif))
    if eos_byte_array[0] != 0x80:
        raise IllegalKey()
    hex_bytea_array = eos_byte_array[1:33]
    return hex_bytea_array


def sign(wfi, trx):
    pri = get_private_ket_by_wif(wfi)
    sha = hashlib.sha256(trx)
    trx = sha.digest()
    pri = bytes(pri)
    ll = ctypes.cdll.LoadLibrary
    try:
        if get_res_path("uECC"):
            libuecc = pkg_resources.resource_filename(__name__, "../" + get_res_path("uECC.cpython"))
        else:
            raise CantFindResInPack
    except:
        libuecc = './uECC.so'

    libuecc = ll(libuecc)
    c_uint_array = c_uint8 * 64
    c_uint_array32 = c_uint8 * 32
    signature = c_uint_array(0)
    c_trx = c_uint_array32(0)
    c_pri = c_uint_array32(0)
    for i in range(32):
        c_trx[i] = trx[i]
        c_pri[i] = pri[i]

    recId = libuecc.uECC_sign_forbc(c_pri, c_trx, signature)
    if recId == -1:
        raise CantFindRecId
    print(recId)
    bin = bytearray()
    binlen = 65 + 4
    headerBytes = recId + 27 + 4
    bin.append(headerBytes)
    bin.extend(bytearray(signature))
    temp = bytearray()
    temp.extend(bin[0:65])
    temp.append(75)
    temp.append(49)
    rmd160 = hashlib.new("rmd160")
    rmd160.update(temp)
    bin.extend(rmd160.digest())
    sig = str(base58.b58encode(bytes(bin)))[2:-1]
    sig = "SIG_K1_" + sig
    return sig



from eosiopy.eosioparams import EosioParams
from eosiopy.exception import ErrInputParams
from eosiopy.nodenetwork import NodeNetwork


class RawinputParams(object):

    def __init__(self, action=None, args=None, code=None, authorization=None):
        self.params_actions_list=list()
        if action and args and code and authorization:
            self.add(action, args, code, authorization)

    def add(self, action, args, code, authorization):
        raw_params = dict()
        raw_params["action"] = action
        raw_params["args"] = args
        raw_params["code"] = code

        try:
            actor = authorization.split("@")[0]
            permission = authorization.split("@")[1]
        except:
            raise ErrInputParams()
        _action_obj = {
            "account": code,
            "authorization": [
                {
                    "actor": actor,
                    "permission": permission
                }
            ],
            "data": self.get_bin(raw_params),
            "name": action
        }
        self.params_actions_list.append(_action_obj)
        return self

    def get_bin(self, json_data):
        return NodeNetwork.json_to_abi(json_data=json_data)["binargs"]






if __name__ == "__main__":
    raw = RawinputParams("transfer", {
        "from": "eosio.token",
        "memo": "dd",
        "quantity": "20.0000 EOS",
        "to": "eosio"
    }, "eosio.token", "eosio.token@active")
    param=EosioParams(raw.params_actions_list)
    print(raw.params_actions_list)




eosio_api_dict = {
    "get_info": "/v1/chain/get_info",
    "get_block": "/v1/chain/get_block",
    "push_transaction": "/v1/chain/push_transaction",
    "abi_json_to_bin": "/v1/chain/abi_json_to_bin",
    "get_account": "/v1/chain/get_account",
    "get_accounts": "/v1/history/get_key_accounts",
    "get_currency_balance": "/v1/chain/get_currency_balance"
}



# -*- coding: utf-8 -*-

from distutils.core import setup, Extension

uECC = Extension('uECC',["eosiopy/uECC.c"])
VERSION = (0, 1, 3)
__version__ = VERSION
__versionstr__ = '.'.join(map(str, VERSION))

f = open('README.md', "r")
long_description = f.read().strip()
f.close()
setup(
    name='eosiopy',
    version=__versionstr__,
    description='Python library of the EOS.IO project.',
    long_description=long_description,
    url='https://github.com/eosmoto/eosiopy',
    author='eosmoto',
    author_email="eosmoto@163.com",
    license='GPL-3.0',
    package_data={'': ['*.so', '*.c', '*.h']},
    ext_modules = [uECC],
    packages=['eosiopy'],
    include_package_data=True,
    install_requires=('requests', 'base58'),
    zip_safe=False,
    keywords=['eos', 'eosio', 'eosiopy', 'eospython', 'pythoneos', 'python eos', 'eos python'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'

    ],
)



import requests

from eosiopy.config import eosio_config


class NodeNetwork(object):
    @staticmethod
    def get_info():
        res = requests.get(eosio_config.url_port + eosio_config.get_info)
        return res.json()

    @staticmethod
    def node_post_base(url, json_data):
        res = requests.post(eosio_config.url_port + url, json_data=json_data)
        return res.json()

    @staticmethod
    def get_block(block_num_or_id):
        res = requests.post(eosio_config.url_port + eosio_config.get_block,
                            json={"block_num_or_id": block_num_or_id})
        return res.json()

    @staticmethod
    def get_info_block():
        res = NodeNetwork.get_info()
        res.update(NodeNetwork.get_block(res["last_irreversible_block_num"]))
        return res

    @staticmethod
    def push_transaction(json_data):
        res = requests.post(eosio_config.url_port + eosio_config.push_transaction, json=json_data)
        return res.json()

    @staticmethod
    def json_to_abi(json_data):
        res = requests.post(eosio_config.url_port + eosio_config.abi_json_to_bin, json=json_data)
        return res.json()

    @staticmethod
    def get_account(json_data):
        res = requests.post(eosio_config.url_port + eosio_config.get_account, json=json_data)
        return res.json()

    @staticmethod
    def get_accounts(json_data):
        res = requests.post(eosio_config.url_port + eosio_config.get_accounts, json=json_data)
        return res.json()

    @staticmethod
    def get_currency_balance(json_data):
        res = requests.post(eosio_config.url_port + eosio_config.get_currency_balance, json=json_data)
        return res.json()



class IllegalKey(Exception):
    pass


class CantFindRecId(Exception):
    pass


class CantFindResInPack(Exception):
    pass


class ErrInputParams(Exception):
    def __str__(self):
        return "Please check the input Should be like this 'eosio.token@active'"



from eosiopy.eosioapi import eosio_api_dict


class EosioConfig(object):
    def __init__(self, url="http://127.0.0.1", port=8888):
        self.url = url
        self.port = port
        self.MAX_NAME_IDX = 12
        self.expiration = 30
        for k, v in eosio_api_dict.items():
            setattr(self, k, v)

    def set_url(self, url):
        self.url = url

    def set_port(self, port):
        self.port = port

    @property
    def url_port(self):
        return self.url + ":" + str(self.port)


eosio_config = EosioConfig()
